//---------------------------------------------------------------------------
#include "GO.h"
#include <stdlib.h>
//---------------------------------------------------------------------------
// HORIZ, DOWNLEFT, DOWNRIGHT, VERT
int Dx[4] = {1, -1, 1, 0}, Dy[4] = {0, 1, 1, 1};

bool GameWon;

int WinningX, WinningY, WinningDx, WinningDy;

//Importance of attack (1..16)
int AttackFactor = 4;

// Value of having 0,1,2,3,4,5 pieces in line
int Weight[6] = {0, 0, 4, 20, 100, 500};

typedef int LineType[2]; // number of and CROSS and NOUGHT in line
typedef int ValueType[2]; // value of square for each player

BoardType Board[N][N]; // the board
int Player; // CROSS or NOUGHT

int TotalLines;

// number of pieces in each of all posible lines
LineType Line[4][N][N];
// value of square for each player
ValueType Value[N][N];

void ResetGame()
{
  for (int i = 0; i < N; i++)
    for (int j = 0; j < N; j++)
    {
      Board[i][j] = EMPTY;
      for (int c = CROSS; c <= NOUGHT; c++)
      {
        Value[i][j][c] = 0;
        for (int d = 0; d < 4; d++)
          Line[d][i][j][c] = 0;
      }
    }
  TotalLines = 2 * N * (N - 4) + 2 * (N - 4) * (N - 4); // Total number of lines
  GameWon = false;
}
//---------------------------------------------------------------------------

// Adds one to the number of pieces in a line
void Add(int& Num)
{
  Num++; // adds one to the number
  if (Num == 1) // if it is the first piece in the line,
    TotalLines--; //then opponent can't use it anymore
  else
  if (Num == 5)
    GameWon = true;
}
//---------------------------------------------------------------------------

// Updates the value of a square for each player,
// taking into account that player has placed
// an extra piece in the square
void Update(LineType& Lin, ValueType& Valu)
{
  int Opponent = 1 - Player;
  // If the opponent has no pieces in the line, then update the value for palyer
  if (Lin[Opponent] == 0)
    Valu[Player] += (Weight[Lin[Player] + 1] - Weight[Lin[Player]]);
  else
    if (Lin[Player] == 1)
      Valu[Opponent] -= Weight[Lin[Opponent]]; //???
}
//---------------------------------------------------------------------------

void MakeMove(int x, int y)
{
// Each square of the board is the part of 20 different lines
// Function adds one to the number of piecs in each of these lines. Then it updates
// the value for each of the 5 squares in each of the 20 lines

  int d;

  // For all of 4 directions
  for (d = 0; d < 4; d++)
  {
    int dx = Dx[d], dy = Dy[d]; // direction of line
    // For all lines, passing through the square
    for (int k = 0; k < 5; k++)
    {
      int x1 = x - dx * k, y1 = y - dy * k; // starting point
      int x2 = x1 + dx * 4, y2 = y1 + dy * 4;
      if (x1 >= 0 && x1 < N && y1 >= 0 && y1 < N &&
          x2 >= 0 && x2 < N && y2 >= 0 && y2 < N) // if line fits to board
      {
        Add(Line[d][x1][y1][Player]);
        if (GameWon)
        {
          WinningX = x1;
          WinningY = y1;
          WinningDx = dx;
          WinningDy = dy;
          goto ExitLoop;
        }
        // Update the value for the 5 squares in the line
        for (int l = 0; l < 5; l++)
          Update(Line[d][x1][y1], Value[x1 + dx * l][y1 + dy * l]);
      }
    }
  }

  ExitLoop:

  Board[x][y] = Player;
}
//---------------------------------------------------------------------------

void FindMove(int& x, int& y)
{
  int Max = 0;

  // The evaluation for a suare is simply the value of the square
  // for the player (attack points) plus the value for the opponent (defence points)
  // Attack is more important than defence, since its is better to get 5 in line
  // than to prevent the opponent from getting it

  for (int i = 0; i < N; i++)
    for (int j = 0; j < N; j++)
      if (Board[i][j] == EMPTY)
      {
        int Valu = Value[i][j][Player] * (16 + AttackFactor) / 16 +
                   Value[i][j][1 - Player] + random(4);
        if (Valu > Max)
        {
          x = i;
          y = j;
          Max = Valu;
        }
      }
 }
//---------------------------------------------------------------------------

